function [aa_id,norm_aa_index,seq_name,seq,seq_len,signal]=GetRawSignal(aa_filename,fasta_filename)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Chua Gek Huey
% Library: HuaySignalLibrary
% Function: GetRawSignal
% Usage : [aa_id,norm_aa_index,seq_name,seq,seq_len,signal]=GetRawSignal(aa_filename,fasta_filename)
% Example: [aa_id,norm_aa_index,seq_name,seq,seq_len,signal]=GetRawSignal('test.txt','test.fasta')
% Purpose:  Read in wanted indices from 'aa_filename' and store accession
%           into aa_id and values of these indices into norm_aa_index
%           Read raw sequences read from 'fasta_filename' and store into
%           seq_name, seq, seq_len
%           Convert sequences to raw signals using these indices and store
%           in signal
% Internal Functions: GetIndexKey, fasta, signal
% Last Edited: 04 Aug 2005
% See Also: GetSignalOneStop, ExtractGetSignal, plotwave, EuclideanDistance
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

if nargin~=2
    disp('Usage:[aa_id,norm_aa_index,seq_name,seq,seq_len,signal]=GetRawSignal(aa_filename,fasta_filename)');
    return;
end

[aa_id,norm_aa_index]=GetIndexKey(aa_filename);
index_num=size(norm_aa_index,1);
[seq,seq_len,seq_name]=fasta(fasta_filename);
signal=Get_aa_index(seq,seq_len,norm_aa_index,index_num);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal Function: GetIndexKey
% Usage:    [aa_id,norm_aa_index]=GetIndexKey(filename);
% Purpose:  Read from 'indices.txt' to get all the 484 indices; 
%           ('indices.txt' includes the AAIndex accession number and their
%           corresponding 20 normalized values.To get the actual values of these indices,
%           please go to http://www.genome.jp/aaindex/ or ask for file 'aaindex1.txt' from 
%           Gek Huey)
%           Read from input 'filename' to get wanted index accession number;
%           Store wanted accession number in aa_id, wanted indices into norm_aa_index 
% File Format for 'filename':
%   ANDN920101 alpha-CH chemical shifts (Andersen et al., 1992)
%   ARGP820101 Hydrophobicity index (Argos et al., 1982)
%   (Note: the most important is the accession number, the description that 
%           follows are not read in, they are just for reference purposes.)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [aa_id,norm_aa_index]=GetIndexKey(filename)

if nargin<1
    disp('Usage:[aa_id,norm_aa_index]=GetIndexKey(filename)');
    return;
end

%Get wanted AAIndex accession number from filename
fid=fopen(filename,'rt');
if (fid==-1)
    disp('Error: cannot open input file');
    return;
end    

index_num=0;
while ~feof(fid)
    tline=fgetl(fid);
    [a,b]=strtok(tline);
    if ~isempty(a)
        index_num=index_num+1;
        aa_id(index_num)=cellstr(a);
    end
end
[wanted_id_s,wanted_id_pos]=sort(aa_id);
fclose(fid);

%Get all 484 indices
fid=fopen('indices.txt','rt');
if (fid==-1)
    disp('Error: cannot find ''indices.txt''\n');
    return;
end    
total_index_num=0;
while ~feof(fid)
    tline=fgetl(fid);
    [name,tline]=strtok(tline);
    if ~isempty(name)
        total_index_num=total_index_num+1;
        id(total_index_num)=cellstr(name);
        a=sscanf(tline,'%f');
        for j=1:20
            aa_index(total_index_num,j)=a(j);
        end
    end
end
[id_s,id_pos]=sort(id);
fclose(fid);

%Store wanted index values into norm_aa_index
j=0;
i=0;
while (i<index_num)
    i=i+1;
    query=wanted_id_s(i);
    while (j<total_index_num)
        j=j+1;
        if (strcmp(query,id_s(j))==1)
            norm_aa_index(wanted_id_pos(i),1:20)=aa_index(id_pos(j),1:20);
            break;
        end
    end
end

% Check if all accession numbers have values
if (i~=index_num)
    disp('Warning: not all accession numbers are read\n');
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal Function: fasta
% Usage: [STRING,LEN,NAME] = fasta(filename)
% Purpose: Read sequences from file in fasta format
% Author: Li Kuo Bin
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [STRING,LEN,NAME] = fasta(filename)
% Read protein sequences in FASTA format
%
% [STRING,LEN] = fasat('filename_in_FASTA');
%
% 

%filename = input('Enter name of file: ', 's');

if nargin <1
   disp('Usage: [STRING,LEN,NAME] = fasta(''filename_in_fasta_format'')')
   return
end

%filename='human_protein_sequences.fasta';
fid = fopen(filename);
flag=0;
id=1;
while ~feof(fid)
    tline=fgetl(fid);
    if ~isempty(tline)
    if (tline(1)~='>')
        if (flag==0)
            flag=1;
            seq_tmp=tline;
        else
            seq_tmp=strcat(seq_tmp,tline);
        end
    else
        if (flag==1)
            LEN(id)=length(seq_tmp);
            STRING(id,1:LEN(id))=seq_tmp(1:LEN(id));
            id=id+1;
        end
        NAME(id)=cellstr(tline);
        flag=0;
    end
    end
end

LEN(id)=length(seq_tmp);
STRING(id,1:LEN(id))=seq_tmp(1:LEN(id));

fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal Function: Get_aa_index
% Usage: [seq_signal] = Get_aa_index(s,l,aa_index,index_num)
% Purpose: Convert sequences to raw signals
% Author: Chua Gek Huey
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [seq_signal] = Get_aa_index(s,l,aa_index,index_num)

if nargin <1
   disp('Usage: [seq_signal] = Get_aa_index(s,l,aa_index,no. of indices)')
   return
end


for i=1:index_num
    for j=1:20
        a(j)=aa_index(i,j);
    end
    for k=1:size(s)
    for j=1:l(k)
        switch (upper(s(k,j)))
        	case 'A'
	            signal =  a(1);
        	case 'R'
        	     signal =  a(2);
        	case 'N'
        	     signal =  a(3);
        	case 'D'
        	     signal =  a(4);
        	case 'C'
        	     signal =  a(5);
        	case 'Q'
        	     signal =  a(6);
        	case 'E'
        	     signal =  a(7);
        	case 'G'
        	     signal =  a(8);
        	case 'H'
        	     signal =  a(9);
        	case 'I'
        	     signal =  a(10);
           	case 'L'
	            signal =  a(11);
        	case 'K'
        	     signal =  a(12);
        	case 'M'
        	     signal =  a(13);
        	case 'F'
        	     signal =  a(14);
        	case 'P'
        	     signal =  a(15);
        	case 'S'
        	     signal =  a(16);
        	case 'T'
        	     signal =  a(17);
        	case 'W'
        	     signal =  a(18);
        	case 'Y'
        	     signal =  a(19);
        	case 'V'
        	     signal =  a(20);
            otherwise
                signal=1000.;
         end
        seq_signal(k,i,j)=signal;
    end
end
end

