function [aa_id,norm_aa_index,seq_name,seq,seq_len,signal,sig_win,wave]=GetSignalOneStop(aa_filename,fasta_filename,win,level,choice,wavelet)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Author: Chua Gek Huey
% Library: HuaySignalLib
% Function: GetSignalOneStop
% Example: [aa_id,norm_aa_index,seq_name,seq,seq_len,signal,sig_win,wave]=GetSignalOneStop('test.txt','test.fasta',1,2,'a','db10')
% Usage: [aa_id,norm_aa_index,seq_name,seq,seq_len,signal,sig_win,wave]
%        =GetSignalOneStop(aa_filename,fasta_filename,win,level,choice,wavelet)
% Purpose:  Get all AA indices wanted from 'aa_filename', store accession
%           into aa_id and values into norm_aa_index
%           Read in sequences from 'fasta_filename' and store in seq_name,
%           seq and seq_len
%           Obtain raw signal with all the AA indices
%           Perform windowing with size 'win' and store in sig_win
%           Perform wavelet transformation and store in wave
%           The coefficients stored in wave is based on 'level' and
%           'choice'.
% Output variables: (size)
%   aa_id = [no. of indices] (cell array)
%   norm_aa_index = [no. of indices][20](float)
%   seq = [no. of seq][len of seq] (char)
%   seq_len = [no. of seq] (integer)
%   seq_name = [no of seq] (cellstr)
%   sig_win = [no. of seq][no of indices][len of seq] (float)
%   signal = [no. of seq][no of indices][len of seq] (float)
%   wave = [no. of seq][no of indices][len of seq] (float)
% Input variables:
%   aa_filename: file containing accession numbers of wanted indices
%   fasta_filename: file containing sequences in fasta format
%   win = size of window (default = 1)
%   level = level of wavelet coefficients wanted (default =2)
%   choice = 'a' for approximation coefficients and 'd' for detail
%               coefficients wanted (default='a')
%   wavelet = mother wavelet (default='db10'
% Internal functions: GetIndexKey, GetAASignal, fasta, wavelevel, wave,
% SigWin, Get_aa_index, approximations
% Last Edited: 04 Aug 2005
% See Also: GetRawSignal, ExtractGetSignal, plotwave, EuclideanDistance
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<2
    disp('Usage:[aa_id,norm_aa_index,seq_name,seq,seq_len,signal,sig_win,wave]=GetSignalOneStop(aa_filename,fasta_filename,win,level,choice,wavelet)');
    return;
elseif nargin==2
    win=1;
    level=2;
    choice='a';
    wavelet='db10';    
elseif nargin==3
    level=2;
    choice='a';
    wavelet='db10';    
elseif nargin==4
    choice='a';
    wavelet='db10';    
elseif nargin==5
    wavelet='db10';    
end

[aa_id,norm_aa_index]=GetIndexKey(aa_filename);
[seq_name,seq,seq_len,signal,sig_win,wave]=GetAASignal(fasta_filename,norm_aa_index,win,level,choice,wavelet);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal Function: GetIndexKey
% Usage:    [aa_id,norm_aa_index]=GetIndexKey(filename);
% Purpose:  Read from 'indices.txt' to get all the 484 indices; 
%           ('indices.txt' includes the AAIndex accession number and their
%           corresponding 20 normalized values.To get the actual values of these indices,
%           please go to http://www.genome.jp/aaindex/ or ask for file 'aaindex1.txt' from 
%           Gek Huey)
%           Read from input 'filename' to get wanted index accession number;
%           Store wanted accession number in aa_id, wanted indices into norm_aa_index 
% File Format for 'filename':
%   ANDN920101 alpha-CH chemical shifts (Andersen et al., 1992)
%   ARGP820101 Hydrophobicity index (Argos et al., 1982)
%   (Note: the most important is the accession number, the description that 
%           follows are not read in, they are just for reference purposes.)
% Last Edited: 04 Aug 2005
% See Also: GetSignalOneStop, ExtractGetSignal
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [aa_id,norm_aa_index]=GetIndexKey(filename)

if nargin<1
    disp('Usage:[aa_id,norm_aa_index]=GetIndexKey(filename)');
    return;
end

%Get wanted AAIndex accession number from filename
fid=fopen(filename,'rt');
if (fid==-1)
    disp('Error: cannot open input file');
    return;
end    

index_num=0;
while ~feof(fid)
    tline=fgetl(fid);
    [a,b]=strtok(tline);
    if ~isempty(a)
        index_num=index_num+1;
        aa_id(index_num)=cellstr(a);
    end
end
[wanted_id_s,wanted_id_pos]=sort(aa_id);
fclose(fid);

%Get all 484 indices
fid=fopen('indices.txt','rt');
if (fid==-1)
    disp('Error: cannot find ''indices.txt''\n');
    return;
end    
total_index_num=0;
while ~feof(fid)
    tline=fgetl(fid);
    [name,tline]=strtok(tline);
    if ~isempty(name)
        total_index_num=total_index_num+1;
        id(total_index_num)=cellstr(name);
        a=sscanf(tline,'%f');
        for j=1:20
            aa_index(total_index_num,j)=a(j);
        end
    end
end
[id_s,id_pos]=sort(id);
fclose(fid);

%Store wanted index values into norm_aa_index
j=0;
i=0;
while (i<index_num)
    i=i+1;
    query=wanted_id_s(i);
    while (j<total_index_num)
        j=j+1;
        if (strcmp(query,id_s(j))==1)
            norm_aa_index(wanted_id_pos(i),1:20)=aa_index(id_pos(j),1:20);
            break;
        end
    end
end

% Check if all accession numbers have values
if (i~=index_num)
    disp('Warning: not all accession numbers are read\n');
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function: GetAASignal
% Usage:    [seq_name,seq,seq_len,signal,sig_win,wave]=GetAASignal(filename,norm_aa_index,win,level,choice,wavelet)
% Purpose:  Read in filename in fasta format and store in seq_name, seq, seq_len
%           Convert sequences to raw signals with indices stored in 'norm_aa_index'
%           and store in signal
%           Perform windowing with size 'win' on signal and store in sig_win
%           Perform wavelet transformation and get coefficients base on
%           'level' and 'choice' where 'choice'='a' means approximation
%           coefficients and 'choice'='d' means detail coefficients
% Internal Functions: fasta, wavelevel,  wave, approximations
% Input variables:
%   filename: file containing sequences in fasta format
%   norm_aa_index: the indices
%   win: the size of window used to smooth the raw signal (default=1)
%   level: the level of coefficients to save into wave (defalut=2)
%   choice: 'a' for approximation coefficients and 'd' for detail
%   coefficients (default='a')
%   wavelet: the mother wavelet used for wavelet transformation
%   (default='db10');
% Author: Chua Gek Huey
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function [seq_name,seq,seq_len,signal,sig_win,wave]=GetAASignal(filename,norm_aa_index,win,level,choice,wavelet)

if nargin<2
   disp('Usage: function [seq_name,seq,seq_len,signal,sig_win,wave]=GetAASignal(filename,norm_aa_index,win,level,choice,wavelet)')
   return
end

if nargin==2
    win=1;
    level=2;
    choice='a';
    wavelet='db10';
elseif nargin==3
    level=2;
    choice='a';
    wavelet='db10';
elseif nargin==4
    choice='a';
    wavelet='db10';
elseif nargin==5
    wavelet='db10';
end


if (choice~='a' & choice~='d')
    disp('Error: choice must be ''a''(for approximation) or ''d''(for detail)');
    return;
end

if (level>4 | level<1)
    disp('Error: minimum level is 1 and maximum level is 4');
    return;
end

index_num=size(norm_aa_index,1);
[seq,seq_len,seq_name]=fasta(filename);
signal=Get_aa_index(seq,seq_len,norm_aa_index,index_num);
for i=1:index_num
    for j=1:size(seq)
        for k=1:seq_len(j)
            tmp_sig(j,k)=signal(j,i,k);
        end
    end
    tmp_win=SigWin(tmp_sig,seq_len,win);
    tmp_wave=wavelevel(wavelet,tmp_win,seq_len,level,choice);
    for j=1:size(seq)
        for k=1:seq_len(j)
            sig_win(j,i,k)=tmp_win(j,k);
            wave(j,i,k)=tmp_wave(j,k);
        end
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal Function: fasta
% Usage: [STRING,LEN,NAME] = fasta(filename)
% Purpose: Read sequences from file in fasta format
% Author: Li Kuo Bin
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [STRING,LEN,NAME] = fasta(filename)
% Read protein sequences in FASTA format
%
% [STRING,LEN] = fasat('filename_in_FASTA');
%
% 

%filename = input('Enter name of file: ', 's');

if nargin <1
   disp('Usage: [STRING,LEN,NAME] = fasta(''filename_in_fasta_format'')')
   return
end

%filename='human_protein_sequences.fasta';
fid = fopen(filename);
flag=0;
id=1;
while ~feof(fid)
    tline=fgetl(fid);
    if ~isempty(tline)
    if (tline(1)~='>')
        if (flag==0)
            flag=1;
            seq_tmp=tline;
        else
            seq_tmp=strcat(seq_tmp,tline);
        end
    else
        if (flag==1)
            LEN(id)=length(seq_tmp);
            STRING(id,1:LEN(id))=seq_tmp(1:LEN(id));
            id=id+1;
        end
        NAME(id)=cellstr(tline);
        flag=0;
    end
    end
end

LEN(id)=length(seq_tmp);
STRING(id,1:LEN(id))=seq_tmp(1:LEN(id));

fclose(fid);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal Function: Get_aa_index
% Usage: [seq_signal] = Get_aa_index(s,l,aa_index,index_num)
% Purpose: Convert sequences to raw signals
% Author: Chua Gek Huey
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [seq_signal] = Get_aa_index(s,l,aa_index,index_num)

if nargin <1
   disp('Usage: [seq_signal] = Get_aa_index(s,l,aa_index,no. of indices)')
   return
end


for i=1:index_num
    for j=1:20
        a(j)=aa_index(i,j);
    end
    for k=1:size(s)
    for j=1:l(k)
        switch (upper(s(k,j)))
        	case 'A'
	            signal =  a(1);
        	case 'R'
        	     signal =  a(2);
        	case 'N'
        	     signal =  a(3);
        	case 'D'
        	     signal =  a(4);
        	case 'C'
        	     signal =  a(5);
        	case 'Q'
        	     signal =  a(6);
        	case 'E'
        	     signal =  a(7);
        	case 'G'
        	     signal =  a(8);
        	case 'H'
        	     signal =  a(9);
        	case 'I'
        	     signal =  a(10);
           	case 'L'
	            signal =  a(11);
        	case 'K'
        	     signal =  a(12);
        	case 'M'
        	     signal =  a(13);
        	case 'F'
        	     signal =  a(14);
        	case 'P'
        	     signal =  a(15);
        	case 'S'
        	     signal =  a(16);
        	case 'T'
        	     signal =  a(17);
        	case 'W'
        	     signal =  a(18);
        	case 'Y'
        	     signal =  a(19);
        	case 'V'
        	     signal =  a(20);
            otherwise
                signal=mean(a);
                disp('unknown aa');
         end
        seq_signal(k,i,j)=signal;
    end
end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal Function: SigWin
% Usage: [sig_win]=SigWin(sig_sum,seq_len,win)
% Purpose: Perform windowing
% Author: Chua Gek Huey
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [sig_win]=SigWin(sig_sum,seq_len,win)

if nargin<1
    disp('Usage: [sig_win] = SigWin(sig_sum,seq_len,win)');
    return;
end
%win
for i=1:size(sig_sum)
    for j=1:(win-1)/2
        sig_win(i,j)=mean(sig_sum(i,1:max(2*j-1,(win-1)/2)));
    end
    for j=(win-1)/2+1:seq_len(i)-(win-1)/2
        sig_win(i,j)=sum(sig_sum(i,j-(win-1)/2:j+(win-1)/2))/win;
    end
    for j=seq_len(i)-(win-1)/2+1:seq_len(i)
        sig_win(i,j)=mean(sig_sum(i,min(seq_len(i)-(win-1)/2,2*j-seq_len(i)):seq_len(i)));
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal Function: wavelevel
% Usage: [seq_wave]=wavelevel(wavelet,seq_win,seq_len,level,choice)
% Purpose: Get wavelet resconstructed waveform and store into 'seq_wave'
% Author: Chua Gek Huey
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [seq_wave]=wavelevel(wavelet,seq_win,seq_len,level,choice)

if nargin <1
   disp('Usage:[seq_wave]=wavelevel(wavelet,seq_win,seq_len,level,choice)')
   return
end

for i=1:size(seq_win)
    [seq_A1,seq_D1] = wave(wavelet,seq_win(i,1:seq_len(i)));
    clear wave_tmp wave_tmp_n
    if (choice=='a')
        for j=1:seq_len(i)
            wave_tmp(j)=seq_A1(level,j);
        end
    end
    if (choice=='d')
        for j=1:seq_len(i)
            wave_tmp(j)=seq_D1(level,j);
        end
    end
    
    %Normalize
    wave_tmp_n=zscore(wave_tmp);
    
    for j=1:seq_len(i)
        seq_wave(i,j)=wave_tmp_n(j);
    end
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal Function: wave
% Usage: [seq_A,seq_D]=wave(waveselect,a);
% Purpose: Perform wavelet transformation
% Author: Arun Krishnan
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [seq_A,seq_D]=wave(waveselect,a);

if nargin <1
   disp('Usage: [seq_A] = wave(waveselect,a)');
   return
end

%Deconstruction
%==============
[seq_C,seq_L] = wavedec(a,4,waveselect);

%Reconstruction
[seq_A,seq_D] = approximations(seq_C,seq_L,4,waveselect);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Internal Function: approximations
% Usage: [x y] = approximations(C,L,r);
% Purpose: Returns the wavelet approximation and detail coefficients for
%           all the sacles in a large array
% Author: Arun Krishnan
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% [C L] are obtained from wavedec
% r is the maximum level of deconstruction
% wname  = wavelet name ; e.g. 'db1'
function [x, y] = approximations(C,L,r,wname)

y=[]; x=[];
for i=1:1:r
x =  [x;wrcoef('a',C,L,wname,i)];
y = [y;wrcoef('d',C,L,wname,i)];
end